@@ -1,18 +1,17 @@
 use std::fmt::{self, Debug, Formatter};
 
+use log::trace;
 use url::Url;
 
-use core::source::{Source, SourceId};
-use core::GitReference;
-use core::{Dependency, Package, PackageId, Registry, Summary};
-use util::Config;
-use util::errors::CargoResult;
-use util::hex::short_hash;
-use sources::PathSource;
-use sources::git::utils::{GitRemote, GitRevision};
+use crate::core::source::{MaybePackage, Source, SourceId};
+use crate::core::GitReference;
+use crate::core::{Dependency, Package, PackageId, Summary};
+use crate::sources::git::utils::{GitRemote, GitRevision};
+use crate::sources::PathSource;
+use crate::util::errors::CargoResult;
+use crate::util::hex::short_hash;
+use crate::util::Config;
 
-/* TODO: Refactor GitSource to delegate to a PathSource
- */
 pub struct GitSource<'cfg> {
     remote: GitRemote,
     reference: GitReference,
@@ -24,7 +23,7 @@ pub struct GitSource<'cfg> {
 }
 
 impl<'cfg> GitSource<'cfg> {
-    pub fn new(source_id: &SourceId, config: &'cfg Config) -> CargoResult<GitSource<'cfg>> {
+    pub fn new(source_id: SourceId, config: &'cfg Config) -> CargoResult<GitSource<'cfg>> {
         assert!(source_id.is_git(), "id is not git, id={}", source_id);
 
         let remote = GitRemote::new(source_id.url());
@@ -38,7 +37,7 @@ impl<'cfg> GitSource<'cfg> {
         let source = GitSource {
             remote,
             reference,
-            source_id: source_id.clone(),
+            source_id,
             path_source: None,
             rev: None,
             ident,
@@ -62,7 +61,8 @@ impl<'cfg> GitSource<'cfg> {
 
 fn ident(url: &Url) -> CargoResult<String> {
     let url = canonicalize_url(url)?;
-    let ident = url.path_segments()
+    let ident = url
+        .path_segments()
         .and_then(|mut s| s.next_back())
         .unwrap_or("");
 
@@ -71,36 +71,36 @@ fn ident(url: &Url) -> CargoResult<String> {
     Ok(format!("{}-{}", ident, short_hash(&url)))
 }
 
-// Some hacks and heuristics for making equivalent URLs hash the same
+// Some hacks and heuristics for making equivalent URLs hash the same.
 pub fn canonicalize_url(url: &Url) -> CargoResult<Url> {
     let mut url = url.clone();
 
-    // cannot-be-a-base-urls are not supported
-    // eg. github.com:rust-lang-nursery/rustfmt.git
+    // cannot-be-a-base-urls (e.g., `github.com:rust-lang-nursery/rustfmt.git`)
+    // are not supported.
     if url.cannot_be_a_base() {
-        bail!(
+        failure::bail!(
             "invalid url `{}`: cannot-be-a-base-URLs are not supported",
             url
         )
     }
 
-    // Strip a trailing slash
+    // Strip a trailing slash.
     if url.path().ends_with('/') {
         url.path_segments_mut().unwrap().pop_if_empty();
     }
 
-    // HACKHACK: For github URL's specifically just lowercase
-    // everything.  GitHub treats both the same, but they hash
+    // HACK: for GitHub URLs specifically, just lower-case
+    // everything. GitHub treats both the same, but they hash
     // differently, and we're gonna be hashing them. This wants a more
     // general solution, and also we're almost certainly not using the
-    // same case conversion rules that GitHub does. (#84)
+    // same case conversion rules that GitHub does. (See issue #84.)
     if url.host_str() == Some("github.com") {
         url.set_scheme("https").unwrap();
         let path = url.path().to_lowercase();
         url.set_path(&path);
     }
 
-    // Repos generally can be accessed with or w/o '.git'
+    // Repos can generally be accessed with or without `.git` extension.
     let needs_chopping = url.path().ends_with(".git");
     if needs_chopping {
         let last = {
@@ -114,7 +114,7 @@ pub fn canonicalize_url(url: &Url) -> CargoResult<Url> {
 }
 
 impl<'cfg> Debug for GitSource<'cfg> {
-    fn fmt(&self, f: &mut Formatter) -> fmt::Result {
+    fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         write!(f, "git repo at {}", self.remote.url())?;
 
         match self.reference.pretty_ref() {
@@ -124,14 +124,23 @@ impl<'cfg> Debug for GitSource<'cfg> {
     }
 }
 
-impl<'cfg> Registry for GitSource<'cfg> {
-    fn query(&mut self, dep: &Dependency, f: &mut FnMut(Summary)) -> CargoResult<()> {
-        let src = self.path_source
+impl<'cfg> Source for GitSource<'cfg> {
+    fn query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+        let src = self
+            .path_source
             .as_mut()
-            .expect("BUG: update() must be called before query()");
+            .expect("BUG: `update()` must be called before `query()`");
         src.query(dep, f)
     }
 
+    fn fuzzy_query(&mut self, dep: &Dependency, f: &mut dyn FnMut(Summary)) -> CargoResult<()> {
+        let src = self
+            .path_source
+            .as_mut()
+            .expect("BUG: `update()` must be called before `query()`");
+        src.fuzzy_query(dep, f)
+    }
+
     fn supports_checksums(&self) -> bool {
         false
     }
@@ -139,11 +148,9 @@ impl<'cfg> Registry for GitSource<'cfg> {
     fn requires_precise(&self) -> bool {
         true
     }
-}
 
-impl<'cfg> Source for GitSource<'cfg> {
-    fn source_id(&self) -> &SourceId {
-        &self.source_id
+    fn source_id(&self) -> SourceId {
+        self.source_id
     }
 
     fn update(&mut self) -> CargoResult<()> {
@@ -155,7 +162,7 @@ impl<'cfg> Source for GitSource<'cfg> {
         let db_path = lock.parent().join("db").join(&self.ident);
 
         if self.config.cli_unstable().offline && !db_path.exists() {
-            bail!(
+            failure::bail!(
                 "can't checkout from '{}': you are in the offline mode (-Z offline)",
                 self.remote.url()
             );
@@ -182,12 +189,12 @@ impl<'cfg> Source for GitSource<'cfg> {
             (self.remote.db_at(&db_path)?, actual_rev.unwrap())
         };
 
-        // Don’t use the full hash,
-        // to contribute less to reaching the path length limit on Windows:
-        // https://github.com/servo/servo/pull/14397
-        let short_id = db.to_short_id(actual_rev.clone()).unwrap();
+        // Don’t use the full hash, in order to contribute less to reaching the path length limit
+        // on Windows. See <https://github.com/servo/servo/pull/14397>.
+        let short_id = db.to_short_id(&actual_rev).unwrap();
 
-        let checkout_path = lock.parent()
+        let checkout_path = lock
+            .parent()
             .join("checkouts")
             .join(&self.ident)
             .join(short_id.as_str());
@@ -200,35 +207,45 @@ impl<'cfg> Source for GitSource<'cfg> {
         db.copy_to(actual_rev.clone(), &checkout_path, self.config)?;
 
         let source_id = self.source_id.with_precise(Some(actual_rev.to_string()));
-        let path_source = PathSource::new_recursive(&checkout_path, &source_id, self.config);
+        let path_source = PathSource::new_recursive(&checkout_path, source_id, self.config);
 
         self.path_source = Some(path_source);
         self.rev = Some(actual_rev);
         self.path_source.as_mut().unwrap().update()
     }
 
-    fn download(&mut self, id: &PackageId) -> CargoResult<Package> {
+    fn download(&mut self, id: PackageId) -> CargoResult<MaybePackage> {
         trace!(
-            "getting packages for package id `{}` from `{:?}`",
+            "getting packages for package ID `{}` from `{:?}`",
             id,
             self.remote
         );
         self.path_source
             .as_mut()
-            .expect("BUG: update() must be called before get()")
+            .expect("BUG: `update()` must be called before `get()`")
             .download(id)
     }
 
+    fn finish_download(&mut self, _id: PackageId, _data: Vec<u8>) -> CargoResult<Package> {
+        panic!("no download should have started")
+    }
+
     fn fingerprint(&self, _pkg: &Package) -> CargoResult<String> {
         Ok(self.rev.as_ref().unwrap().to_string())
     }
+
+    fn describe(&self) -> String {
+        format!("Git repository {}", self.source_id)
+    }
+
+    fn add_to_yanked_whitelist(&mut self, _pkgs: &[PackageId]) {}
 }
 
 #[cfg(test)]
 mod test {
-    use url::Url;
     use super::ident;
-    use util::ToUrl;
+    use crate::util::ToUrl;
+    use url::Url;
 
     #[test]
     pub fn test_url_to_path_ident_with_path() {
