@@ -1,11 +1,9 @@
 use std::fs::File;
 use std::io::prelude::*;
 
-use support::paths::CargoPathExt;
-use support::{basic_manifest, execs, project};
-use support::ChannelChanger;
-use support::hamcrest::assert_that;
-use support::registry::Package;
+use crate::support::paths::CargoPathExt;
+use crate::support::registry::Package;
+use crate::support::{basic_manifest, project};
 
 #[test]
 fn invalid1() {
@@ -25,17 +23,17 @@ fn invalid1() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` includes `baz` which is neither a dependency nor another feature
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -59,17 +57,17 @@ fn invalid2() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Features and dependencies cannot have the same name: `bar`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -93,9 +91,9 @@ fn invalid3() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
@@ -103,8 +101,8 @@ Caused by:
   Feature `bar` depends on `baz` which is not an optional dependency.
 Consider adding `optional = true` to the dependency
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -128,9 +126,9 @@ fn invalid4() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 error: failed to select a version for `bar`.
     ... required by package `foo v0.0.1 ([..])`
@@ -140,17 +138,15 @@ the package `foo` depends on `bar`, with features: `bar` but `bar` does not have
 
 
 failed to select a version for `bar` which could resolve this conflict",
-        ),
-    );
+        )
+        .run();
 
     p.change_file("Cargo.toml", &basic_manifest("foo", "0.0.1"));
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("test"),
-        execs()
-            .with_status(101)
-            .with_stderr("error: Package `foo v0.0.1 ([..])` does not have these features: `test`"),
-    );
+    p.cargo("build --features test")
+        .with_status(101)
+        .with_stderr("error: Package `foo v0.0.1 ([..])` does not have these features: `test`")
+        .run();
 }
 
 #[test]
@@ -172,17 +168,17 @@ fn invalid5() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Dev-dependencies are not allowed to be optional: `bar`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -203,17 +199,17 @@ fn invalid6() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("foo"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build --features foo")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `foo` requires a feature of `bar` which is not a dependency
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -235,17 +231,17 @@ fn invalid7() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("foo"),
-        execs().with_status(101).with_stderr(
+    p.cargo("build --features foo")
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `foo` requires a feature of `bar` which is not a dependency
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -269,12 +265,10 @@ fn invalid8() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("foo"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] feature names may not contain slashes: `foo/bar`"),
-    );
+    p.cargo("build --features foo")
+        .with_status(101)
+        .with_stderr("[ERROR] feature names may not contain slashes: `foo/bar`")
+        .run();
 }
 
 #[test]
@@ -297,14 +291,13 @@ fn invalid9() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build").arg("--features").arg("bar"),
-                execs().with_stderr("\
+    p.cargo("build --features bar").with_stderr("\
 warning: Package `foo v0.0.1 ([..])` does not have feature `bar`. It has a required dependency with \
 that name, but only optional dependencies can be used as features. [..]
    Compiling bar v0.0.1 ([..])
    Compiling foo v0.0.1 ([..])
     Finished dev [unoptimized + debuginfo] target(s) in [..]s
-"));
+").run();
 }
 
 #[test]
@@ -341,15 +334,14 @@ fn invalid10() {
         .file("bar/baz/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"),
-                execs().with_stderr("\
+    p.cargo("build").with_stderr("\
 warning: Package `bar v0.0.1 ([..])` does not have feature `baz`. It has a required dependency with \
 that name, but only optional dependencies can be used as features. [..]
    Compiling baz v0.0.1 ([..])
    Compiling bar v0.0.1 ([..])
    Compiling foo v0.0.1 ([..])
     Finished dev [unoptimized + debuginfo] target(s) in [..]s
-"));
+").run();
 }
 
 #[test]
@@ -410,12 +402,10 @@ fn no_transitive_dep_feature_requirement() {
         "#,
         )
         .build();
-    assert_that(
-        p.cargo("build"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] feature names may not contain slashes: `bar/qux`"),
-    );
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr("[ERROR] feature names may not contain slashes: `bar/qux`")
+        .run();
 }
 
 #[test]
@@ -449,36 +439,26 @@ fn no_feature_doesnt_build() {
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(format!(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
-    assert_that(
-        p.process(&p.bin("foo")),
-        execs().with_stdout(""),
-    );
+        )
+        .run();
+    p.process(&p.bin("foo")).with_stdout("").run();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("bar"),
-        execs().with_stderr(format!(
+    p.cargo("build --features bar")
+        .with_stderr(
             "\
-[COMPILING] bar v0.0.1 ({dir}/bar)
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] bar v0.0.1 ([CWD]/bar)
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
-    assert_that(
-        p.process(&p.bin("foo")),
-        execs().with_stdout("bar\n"),
-    );
+        )
+        .run();
+    p.process(&p.bin("foo")).with_stdout("bar\n").run();
 }
 
 #[test]
@@ -515,36 +495,26 @@ fn default_feature_pulled_in() {
         .file("bar/src/lib.rs", "pub fn bar() {}")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(format!(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] bar v0.0.1 ({dir}/bar)
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] bar v0.0.1 ([CWD]/bar)
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
-    assert_that(
-        p.process(&p.bin("foo")),
-        execs().with_stdout("bar\n"),
-    );
+        )
+        .run();
+    p.process(&p.bin("foo")).with_stdout("bar\n").run();
 
-    assert_that(
-        p.cargo("build").arg("--no-default-features"),
-        execs().with_stderr(format!(
+    p.cargo("build --no-default-features")
+        .with_stderr(
             "\
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
-    assert_that(
-        p.process(&p.bin("foo")),
-        execs().with_stdout(""),
-    );
+        )
+        .run();
+    p.process(&p.bin("foo")).with_stdout("").run();
 }
 
 #[test]
@@ -565,12 +535,10 @@ fn cyclic_feature() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] Cyclic feature dependency: feature `default` depends on itself"),
-    );
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr("[ERROR] cyclic feature dependency: feature `default` depends on itself")
+        .run();
 }
 
 #[test]
@@ -592,7 +560,7 @@ fn cyclic_feature2() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(p.cargo("build"), execs().with_stdout(""));
+    p.cargo("build").with_stdout("").run();
 }
 
 #[test]
@@ -641,18 +609,16 @@ fn groups_on_groups_on_groups() {
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(format!(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -691,18 +657,17 @@ fn many_cli_features() {
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("bar baz"),
-        execs().with_stderr(format!(
+    p.cargo("build --features")
+        .arg("bar baz")
+        .with_stderr(
             "\
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -776,18 +741,16 @@ fn union_features() {
         )
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(format!(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] d2 v0.0.1 ({dir}/d2)
-[COMPILING] d1 v0.0.1 ({dir}/d1)
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] d2 v0.0.1 ([CWD]/d2)
+[COMPILING] d1 v0.0.1 ([CWD]/d1)
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -824,42 +787,34 @@ fn many_features_no_rebuilds() {
         .file("a/src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(format!(
+    p.cargo("build")
+        .with_stderr(
             "\
-[COMPILING] a v0.1.0 ({dir}/a)
-[COMPILING] b v0.1.0 ({dir})
+[COMPILING] a v0.1.0 ([CWD]/a)
+[COMPILING] b v0.1.0 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
     p.root().move_into_the_past();
 
-    assert_that(
-        p.cargo("build").arg("-v"),
-        execs().with_stderr(
+    p.cargo("build -v")
+        .with_stderr(
             "\
 [FRESH] a v0.1.0 ([..]/a)
 [FRESH] b v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 // Tests that all cmd lines work with `--features ""`
 #[test]
 fn empty_features() {
-    let p = project()
-        .file("src/main.rs", "fn main() {}")
-        .build();
+    let p = project().file("src/main.rs", "fn main() {}").build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg(""),
-        execs(),
-    );
+    p.cargo("build --features").arg("").run();
 }
 
 // Tests that all cmd lines work with `--features ""`
@@ -894,13 +849,13 @@ fn transitive_features() {
             baz = []
         "#,
         )
-        .file("bar/src/lib.rs", r#"#[cfg(feature = "baz")] pub fn baz() {}"#)
+        .file(
+            "bar/src/lib.rs",
+            r#"#[cfg(feature = "baz")] pub fn baz() {}"#,
+        )
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("foo"),
-        execs(),
-    );
+    p.cargo("build --features foo").run();
 }
 
 #[test]
@@ -959,7 +914,7 @@ fn everything_in_the_lockfile() {
         .file("d3/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("fetch"), execs());
+    p.cargo("fetch").run();
     let loc = p.root().join("Cargo.lock");
     let mut lockfile = String::new();
     t!(t!(File::open(&loc)).read_to_string(&mut lockfile));
@@ -1026,9 +981,9 @@ fn no_rebuild_when_frobbing_default_feature() {
         .file("a/src/lib.rs", "")
         .build();
 
-    assert_that(p.cargo("build"), execs());
-    assert_that(p.cargo("build"), execs().with_stdout(""));
-    assert_that(p.cargo("build"), execs().with_stdout(""));
+    p.cargo("build").run();
+    p.cargo("build").with_stdout("").run();
+    p.cargo("build").with_stdout("").run();
 }
 
 #[test]
@@ -1077,9 +1032,9 @@ fn unions_work_with_no_default_features() {
         .file("a/src/lib.rs", r#"#[cfg(feature = "f1")] pub fn a() {}"#)
         .build();
 
-    assert_that(p.cargo("build"), execs());
-    assert_that(p.cargo("build"), execs().with_stdout(""));
-    assert_that(p.cargo("build"), execs().with_stdout(""));
+    p.cargo("build").run();
+    p.cargo("build").with_stdout("").run();
+    p.cargo("build").with_stdout("").run();
 }
 
 #[test]
@@ -1104,15 +1059,14 @@ fn optional_and_dev_dep() {
         .file("foo/src/lib.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs().with_stderr(
+    p.cargo("build")
+        .with_stderr(
             "\
 [COMPILING] test v0.1.0 ([..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1133,7 +1087,10 @@ fn activating_feature_activates_dep() {
             a = ["foo/a"]
         "#,
         )
-        .file("src/lib.rs", "extern crate foo; pub fn bar() { foo::bar(); }")
+        .file(
+            "src/lib.rs",
+            "extern crate foo; pub fn bar() { foo::bar(); }",
+        )
         .file(
             "foo/Cargo.toml",
             r#"
@@ -1149,10 +1106,7 @@ fn activating_feature_activates_dep() {
         .file("foo/src/lib.rs", r#"#[cfg(feature = "a")] pub fn bar() {}"#)
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("a").arg("-v"),
-        execs(),
-    );
+    p.cargo("build --features a -v").run();
 }
 
 #[test]
@@ -1217,34 +1171,26 @@ fn dep_feature_in_cmd_line() {
 
     // The foo project requires that feature "some-feat" in "bar" is enabled.
     // Building without any features enabled should fail:
-    assert_that(p.cargo("build"), execs().with_status(101));
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr_contains("[..]unresolved import `bar::test`")
+        .run();
 
     // We should be able to enable the feature "derived-feat", which enables "some-feat",
     // on the command line. The feature is enabled, thus building should be successful:
-    assert_that(
-        p.cargo("build")
-            .arg("--features")
-            .arg("derived/derived-feat"),
-        execs(),
-    );
+    p.cargo("build --features derived/derived-feat").run();
 
     // Trying to enable features of transitive dependencies is an error
-    assert_that(
-        p.cargo("build").arg("--features").arg("bar/some-feat"),
-        execs()
-            .with_status(101)
-            .with_stderr("error: Package `foo v0.0.1 ([..])` does not have these features: `bar`"),
-    );
+    p.cargo("build --features bar/some-feat")
+        .with_status(101)
+        .with_stderr("error: Package `foo v0.0.1 ([..])` does not have these features: `bar`")
+        .run();
 
     // Hierarchical feature specification should still be disallowed
-    assert_that(
-        p.cargo("build")
-            .arg("--features")
-            .arg("derived/bar/some-feat"),
-        execs()
-            .with_status(101)
-            .with_stderr("[ERROR] feature names may not contain slashes: `bar/some-feat`"),
-    );
+    p.cargo("build --features derived/bar/some-feat")
+        .with_status(101)
+        .with_stderr("[ERROR] feature names may not contain slashes: `bar/some-feat`")
+        .run();
 }
 
 #[test]
@@ -1289,10 +1235,7 @@ fn all_features_flag_enables_all_features() {
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--all-features"),
-        execs(),
-    );
+    p.cargo("build --all-features").run();
 }
 
 #[test]
@@ -1331,18 +1274,16 @@ fn many_cli_features_comma_delimited() {
         .file("baz/src/lib.rs", "pub fn baz() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("bar,baz"),
-        execs().with_stderr(format!(
+    p.cargo("build --features bar,baz")
+        .with_stderr(
             "\
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1397,20 +1338,19 @@ fn many_cli_features_comma_and_space_delimited() {
         .file("bap/src/lib.rs", "pub fn bap() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").arg("--features").arg("bar,baz bam bap"),
-        execs().with_stderr(format!(
+    p.cargo("build --features")
+        .arg("bar,baz bam bap")
+        .with_stderr(
             "\
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] ba[..] v0.0.1 ({dir}/ba[..])
-[COMPILING] foo v0.0.1 ({dir})
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] ba[..] v0.0.1 ([CWD]/ba[..])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
-            dir = p.url()
-        )),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1454,50 +1394,46 @@ fn combining_features_and_package() {
         )
         .build();
 
-    assert_that(
-        p.cargo("build -Z package-features --all --features main")
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr_contains(
+    p.cargo("build -Z package-features --all --features main")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr_contains(
             "\
              [ERROR] cannot specify features for more than one package",
-        ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("build -Z package-features --package dep --features main")
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr_contains(
+    p.cargo("build -Z package-features --package dep --features main")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr_contains(
             "\
              [ERROR] cannot specify features for packages outside of workspace",
-        ),
-    );
-    assert_that(
-        p.cargo("build -Z package-features --package dep --all-features")
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr_contains(
+        )
+        .run();
+    p.cargo("build -Z package-features --package dep --all-features")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr_contains(
             "\
              [ERROR] cannot specify features for packages outside of workspace",
-        ),
-    );
-    assert_that(
-        p.cargo("build -Z package-features --package dep --no-default-features")
-            .masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr_contains(
+        )
+        .run();
+    p.cargo("build -Z package-features --package dep --no-default-features")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr_contains(
             "\
              [ERROR] cannot specify features for packages outside of workspace",
-        ),
-    );
+        )
+        .run();
 
-    assert_that(
-        p.cargo("build -Z package-features --all --all-features")
-            .masquerade_as_nightly_cargo(),
-        execs(),
-    );
-    assert_that(
-        p.cargo("run -Z package-features --package bar --features main")
-            .masquerade_as_nightly_cargo(),
-        execs(),
-    );
+    p.cargo("build -Z package-features --all --all-features")
+        .masquerade_as_nightly_cargo()
+        .run();
+    p.cargo("run -Z package-features --package bar --features main")
+        .masquerade_as_nightly_cargo()
+        .run();
 }
 
 #[test]
@@ -1521,17 +1457,18 @@ fn namespaced_invalid_feature() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` includes `baz` which is not defined as a feature
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1555,17 +1492,18 @@ fn namespaced_invalid_dependency() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` includes `crate:baz` which is not a known dependency
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1592,9 +1530,10 @@ fn namespaced_non_optional_dependency() {
         .file("src/main.rs", "")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build")
+        .masquerade_as_nightly_cargo()
+        .with_status(101)
+        .with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
@@ -1602,8 +1541,8 @@ Caused by:
   Feature `bar` includes `crate:baz` which is not an optional dependency.
 Consider adding `optional = true` to the dependency
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1630,10 +1569,7 @@ fn namespaced_implicit_feature() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs(),
-    );
+    p.cargo("build").masquerade_as_nightly_cargo().run();
 }
 
 #[test]
@@ -1660,9 +1596,7 @@ fn namespaced_shadowed_dep() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
@@ -1670,8 +1604,8 @@ Caused by:
   Feature `baz` includes the optional dependency of the same name, but this is left implicit in the features included by this feature.
 Consider adding `crate:baz` to this feature's requirements.
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1698,9 +1632,7 @@ fn namespaced_shadowed_non_optional() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
@@ -1709,8 +1641,8 @@ Caused by:
 Additionally, the dependency must be marked as optional to be included in the feature definition.
 Consider adding `crate:baz` to this feature's requirements and marking the dependency as `optional = true`
 ",
-        ),
-    );
+        )
+        .run();
 }
 
 #[test]
@@ -1737,9 +1669,7 @@ fn namespaced_implicit_non_optional() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs().with_status(101).with_stderr(
+    p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
             "\
 [ERROR] failed to parse manifest at `[..]`
 
@@ -1747,7 +1677,7 @@ Caused by:
   Feature `bar` includes `baz` which is not defined as a feature.
 A non-optional dependency of the same name is defined; consider adding `optional = true` to its definition
 ",
-        ),
+        ).run(
     );
 }
 
@@ -1775,10 +1705,7 @@ fn namespaced_same_name() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build").masquerade_as_nightly_cargo(),
-        execs(),
-    );
+    p.cargo("build").masquerade_as_nightly_cargo().run();
 }
 
 #[test]
@@ -1807,10 +1734,7 @@ fn only_dep_is_optional() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build"),
-        execs(),
-    );
+    p.cargo("build").run();
 }
 
 #[test]
@@ -1846,8 +1770,100 @@ fn all_features_all_crates() {
         .file("bar/src/main.rs", "#[cfg(feature = \"foo\")] fn main() {}")
         .build();
 
-    assert_that(
-        p.cargo("build --all-features --all"),
-        execs(),
-    );
+    p.cargo("build --all-features --all").run();
+}
+
+#[test]
+fn feature_off_dylib() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [workspace]
+            members = ["bar"]
+
+            [package]
+            name = "foo"
+            version = "0.0.1"
+
+            [lib]
+            crate-type = ["dylib"]
+
+            [features]
+            f1 = []
+        "#,
+        )
+        .file(
+            "src/lib.rs",
+            r#"
+            pub fn hello() -> &'static str {
+                if cfg!(feature = "f1") {
+                    "f1"
+                } else {
+                    "no f1"
+                }
+            }
+        "#,
+        )
+        .file(
+            "bar/Cargo.toml",
+            r#"
+            [package]
+            name = "bar"
+            version = "0.0.1"
+
+            [dependencies]
+            foo = { path = ".." }
+        "#,
+        )
+        .file(
+            "bar/src/main.rs",
+            r#"
+            extern crate foo;
+
+            fn main() {
+                assert_eq!(foo::hello(), "no f1");
+            }
+        "#,
+        )
+        .build();
+
+    // Build the dylib with `f1` feature.
+    p.cargo("build --features f1").run();
+    // Check that building without `f1` uses a dylib without `f1`.
+    p.cargo("run -p bar").run();
+}
+
+#[test]
+fn warn_if_default_features() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [dependencies.bar]
+            path = "bar"
+            optional = true
+
+            [features]
+            default-features = ["bar"]
+         "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
+        .file("bar/src/lib.rs", "pub fn bar() {}")
+        .build();
+
+    p.cargo("build")
+        .with_stderr(
+            r#"
+[WARNING] `default-features = [".."]` was found in [features]. Did you mean to use `default = [".."]`?
+[COMPILING] foo v0.0.1 ([CWD])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+            "#.trim(),
+        ).run();
 }
