@@ -1,36 +1,32 @@
-// Copyright 2014-2018 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
-
 #![allow(clippy::default_hash_types)]
 
 use itertools::Itertools;
 use lazy_static::lazy_static;
 use regex::Regex;
-use walkdir::WalkDir;
 use std::collections::HashMap;
 use std::ffi::OsStr;
 use std::fs;
 use std::io::prelude::*;
+use walkdir::WalkDir;
 
 lazy_static! {
-    static ref DEC_CLIPPY_LINT_RE: Regex = Regex::new(r#"(?x)
+    static ref DEC_CLIPPY_LINT_RE: Regex = Regex::new(
+        r#"(?x)
         declare_clippy_lint!\s*[\{(]\s*
         pub\s+(?P<name>[A-Z_][A-Z_0-9]*)\s*,\s*
         (?P<cat>[a-z_]+)\s*,\s*
         "(?P<desc>(?:[^"\\]+|\\(?s).(?-s))*)"\s*[})]
-    "#).unwrap();
-    static ref DEC_DEPRECATED_LINT_RE: Regex = Regex::new(r#"(?x)
+    "#
+    )
+    .unwrap();
+    static ref DEC_DEPRECATED_LINT_RE: Regex = Regex::new(
+        r#"(?x)
         declare_deprecated_lint!\s*[{(]\s*
         pub\s+(?P<name>[A-Z_][A-Z_0-9]*)\s*,\s*
         "(?P<desc>(?:[^"\\]+|\\(?s).(?-s))*)"\s*[})]
-    "#).unwrap();
+    "#
+    )
+    .unwrap();
     static ref NL_ESCAPE_RE: Regex = Regex::new(r#"\\\n\s*"#).unwrap();
     pub static ref DOCS_LINK: String = "https://rust-lang.github.io/rust-clippy/master/index.html".to_string();
 }
@@ -51,19 +47,22 @@ impl Lint {
             name: name.to_lowercase(),
             group: group.to_string(),
             desc: NL_ESCAPE_RE.replace(&desc.replace("\\\"", "\""), "").to_string(),
-            deprecation: deprecation.map(|d| d.to_string()),
+            deprecation: deprecation.map(std::string::ToString::to_string),
             module: module.to_string(),
         }
     }
 
     /// Returns all non-deprecated lints and non-internal lints
-    pub fn usable_lints(lints: impl Iterator<Item=Self>) -> impl Iterator<Item=Self> {
+    pub fn usable_lints(lints: impl Iterator<Item = Self>) -> impl Iterator<Item = Self> {
         lints.filter(|l| l.deprecation.is_none() && !l.is_internal())
     }
 
     /// Returns the lints in a HashMap, grouped by the different lint groups
     pub fn by_lint_group(lints: &[Self]) -> HashMap<String, Vec<Self>> {
-        lints.iter().map(|lint| (lint.group.to_string(), lint.clone())).into_group_map()
+        lints
+            .iter()
+            .map(|lint| (lint.group.to_string(), lint.clone()))
+            .into_group_map()
     }
 
     pub fn is_internal(&self) -> bool {
@@ -73,7 +72,8 @@ impl Lint {
 
 /// Generates the Vec items for `register_lint_group` calls in `clippy_lints/src/lib.rs`.
 pub fn gen_lint_group_list(lints: Vec<Lint>) -> Vec<String> {
-    lints.into_iter()
+    lints
+        .into_iter()
         .filter_map(|l| {
             if l.is_internal() || l.deprecation.is_some() {
                 None
@@ -82,19 +82,24 @@ pub fn gen_lint_group_list(lints: Vec<Lint>) -> Vec<String> {
             }
         })
         .sorted()
+        .collect::<Vec<String>>()
 }
 
 /// Generates the `pub mod module_name` list in `clippy_lints/src/lib.rs`.
 pub fn gen_modules_list(lints: Vec<Lint>) -> Vec<String> {
-    lints.into_iter()
+    lints
+        .into_iter()
         .filter_map(|l| {
-            if l.is_internal() || l.deprecation.is_some() { None } else { Some(l.module) }
+            if l.is_internal() || l.deprecation.is_some() {
+                None
+            } else {
+                Some(l.module)
+            }
         })
         .unique()
-        .map(|module| {
-            format!("pub mod {};", module)
-        })
+        .map(|module| format!("pub mod {};", module))
         .sorted()
+        .collect::<Vec<String>>()
 }
 
 /// Generates the list of lint links at the bottom of the README
@@ -109,35 +114,34 @@ pub fn gen_changelog_lint_list(lints: Vec<Lint>) -> Vec<String> {
             } else {
                 Some(format!("[`{}`]: {}#{}", l.name, DOCS_LINK.clone(), l.name))
             }
-        }).collect()
+        })
+        .collect()
 }
 
 /// Generates the `register_removed` code in `./clippy_lints/src/lib.rs`.
 pub fn gen_deprecated(lints: &[Lint]) -> Vec<String> {
-    itertools::flatten(
-        lints
-            .iter()
-            .filter_map(|l| {
-                l.clone().deprecation.and_then(|depr_text| {
-                    Some(
-                        vec![
-                            "    store.register_removed(".to_string(),
-                            format!("        \"{}\",", l.name),
-                            format!("        \"{}\",", depr_text),
-                            "    );".to_string()
-                        ]
-                    )
-                })
+    lints
+        .iter()
+        .filter_map(|l| {
+            l.clone().deprecation.and_then(|depr_text| {
+                Some(vec![
+                    "    store.register_removed(".to_string(),
+                    format!("        \"{}\",", l.name),
+                    format!("        \"{}\",", depr_text),
+                    "    );".to_string(),
+                ])
             })
-    ).collect()
+        })
+        .flatten()
+        .collect::<Vec<String>>()
 }
 
 /// Gathers all files in `src/clippy_lints` and gathers all lints inside
-pub fn gather_all() -> impl Iterator<Item=Lint> {
+pub fn gather_all() -> impl Iterator<Item = Lint> {
     lint_files().flat_map(|f| gather_from_file(&f))
 }
 
-fn gather_from_file(dir_entry: &walkdir::DirEntry) -> impl Iterator<Item=Lint> {
+fn gather_from_file(dir_entry: &walkdir::DirEntry) -> impl Iterator<Item = Lint> {
     let mut file = fs::File::open(dir_entry.path()).unwrap();
     let mut content = String::new();
     file.read_to_string(&mut content).unwrap();
@@ -145,29 +149,36 @@ fn gather_from_file(dir_entry: &walkdir::DirEntry) -> impl Iterator<Item=Lint> {
     // If the lints are stored in mod.rs, we get the module name from
     // the containing directory:
     if filename == "mod" {
-        filename = dir_entry.path().parent().unwrap().file_stem().unwrap().to_str().unwrap()
+        filename = dir_entry
+            .path()
+            .parent()
+            .unwrap()
+            .file_stem()
+            .unwrap()
+            .to_str()
+            .unwrap()
     }
     parse_contents(&content, filename)
 }
 
-fn parse_contents(content: &str, filename: &str) -> impl Iterator<Item=Lint> {
+fn parse_contents(content: &str, filename: &str) -> impl Iterator<Item = Lint> {
     let lints = DEC_CLIPPY_LINT_RE
         .captures_iter(content)
         .map(|m| Lint::new(&m["name"], &m["cat"], &m["desc"], None, filename));
     let deprecated = DEC_DEPRECATED_LINT_RE
         .captures_iter(content)
-        .map(|m| Lint::new( &m["name"], "Deprecated", &m["desc"], Some(&m["desc"]), filename));
+        .map(|m| Lint::new(&m["name"], "Deprecated", &m["desc"], Some(&m["desc"]), filename));
     // Removing the `.collect::<Vec<Lint>>().into_iter()` causes some lifetime issues due to the map
     lints.chain(deprecated).collect::<Vec<Lint>>().into_iter()
 }
 
 /// Collects all .rs files in the `clippy_lints/src` directory
-fn lint_files() -> impl Iterator<Item=walkdir::DirEntry> {
+fn lint_files() -> impl Iterator<Item = walkdir::DirEntry> {
     // We use `WalkDir` instead of `fs::read_dir` here in order to recurse into subdirectories.
     // Otherwise we would not collect all the lints, for example in `clippy_lints/src/methods/`.
     WalkDir::new("../clippy_lints/src")
         .into_iter()
-        .filter_map(|f| f.ok())
+        .filter_map(std::result::Result::ok)
         .filter(|f| f.path().extension() == Some(OsStr::new("rs")))
 }
 
@@ -184,15 +195,27 @@ pub struct FileChange {
 ///
 /// See `replace_region_in_text` for documentation of the other options.
 #[allow(clippy::expect_fun_call)]
-pub fn replace_region_in_file<F>(path: &str, start: &str, end: &str, replace_start: bool, write_back: bool, replacements: F) -> FileChange where F: Fn() -> Vec<String> {
+pub fn replace_region_in_file<F>(
+    path: &str,
+    start: &str,
+    end: &str,
+    replace_start: bool,
+    write_back: bool,
+    replacements: F,
+) -> FileChange
+where
+    F: Fn() -> Vec<String>,
+{
     let mut f = fs::File::open(path).expect(&format!("File not found: {}", path));
     let mut contents = String::new();
-    f.read_to_string(&mut contents).expect("Something went wrong reading the file");
+    f.read_to_string(&mut contents)
+        .expect("Something went wrong reading the file");
     let file_change = replace_region_in_text(&contents, start, end, replace_start, replacements);
 
     if write_back {
         let mut f = fs::File::create(path).expect(&format!("File not found: {}", path));
-        f.write_all(file_change.new_lines.as_bytes()).expect("Unable to write file");
+        f.write_all(file_change.new_lines.as_bytes())
+            .expect("Unable to write file");
         // Ensure we write the changes with a trailing newline so that
         // the file has the proper line endings.
         f.write_all(b"\n").expect("Unable to write file");
@@ -205,10 +228,10 @@ pub fn replace_region_in_file<F>(path: &str, start: &str, end: &str, replace_sta
 /// * `text` is the input text on which you want to perform the replacement
 /// * `start` is a `&str` that describes the delimiter line before the region you want to replace.
 ///   As the `&str` will be converted to a `Regex`, this can contain regex syntax, too.
-/// * `end` is a `&str` that describes the delimiter line until where the replacement should
-///   happen.  As the `&str` will be converted to a `Regex`, this can contain regex syntax, too.
-/// * If `replace_start` is true, the `start` delimiter line is replaced as well.
-///   The `end` delimiter line is never replaced.
+/// * `end` is a `&str` that describes the delimiter line until where the replacement should happen.
+///   As the `&str` will be converted to a `Regex`, this can contain regex syntax, too.
+/// * If `replace_start` is true, the `start` delimiter line is replaced as well. The `end`
+///   delimiter line is never replaced.
 /// * `replacements` is a closure that has to return a `Vec<String>` which contains the new text.
 ///
 /// If you want to perform the replacement on files instead of already parsed text,
@@ -218,18 +241,16 @@ pub fn replace_region_in_file<F>(path: &str, start: &str, end: &str, replace_sta
 ///
 /// ```
 /// let the_text = "replace_start\nsome text\nthat will be replaced\nreplace_end";
-/// let result = clippy_dev::replace_region_in_text(
-///     the_text,
-///     r#"replace_start"#,
-///     r#"replace_end"#,
-///     false,
-///     || {
-///         vec!["a different".to_string(), "text".to_string()]
-///     }
-/// ).new_lines;
+/// let result = clippy_dev::replace_region_in_text(the_text, r#"replace_start"#, r#"replace_end"#, false, || {
+///     vec!["a different".to_string(), "text".to_string()]
+/// })
+/// .new_lines;
 /// assert_eq!("replace_start\na different\ntext\nreplace_end", result);
 /// ```
-pub fn replace_region_in_text<F>(text: &str, start: &str, end: &str, replace_start: bool, replacements: F) -> FileChange where F: Fn() -> Vec<String> {
+pub fn replace_region_in_text<F>(text: &str, start: &str, end: &str, replace_start: bool, replacements: F) -> FileChange
+where
+    F: Fn() -> Vec<String>,
+{
     let lines = text.lines();
     let mut in_old_region = false;
     let mut found = false;
@@ -264,7 +285,7 @@ pub fn replace_region_in_text<F>(text: &str, start: &str, end: &str, replace_sta
 
     FileChange {
         changed: lines.ne(new_lines.clone()),
-        new_lines: new_lines.join("\n")
+        new_lines: new_lines.join("\n"),
     }
 }
 
@@ -291,7 +312,9 @@ declare_deprecated_lint! {
     "`assert!()` will be more flexible with RFC 2011"
 }
     "#,
-    "module_name").collect();
+        "module_name",
+    )
+    .collect();
 
     let expected = vec![
         Lint::new("ptr_arg", "style", "really long text", None, "module_name"),
@@ -301,7 +324,7 @@ declare_deprecated_lint! {
             "Deprecated",
             "`assert!()` will be more flexible with RFC 2011",
             Some("`assert!()` will be more flexible with RFC 2011"),
-            "module_name"
+            "module_name",
         ),
     ];
     assert_eq!(expected, result);
@@ -312,7 +335,7 @@ fn test_replace_region() {
     let text = "\nabc\n123\n789\ndef\nghi";
     let expected = FileChange {
         changed: true,
-        new_lines: "\nabc\nhello world\ndef\nghi".to_string()
+        new_lines: "\nabc\nhello world\ndef\nghi".to_string(),
     };
     let result = replace_region_in_text(text, r#"^\s*abc$"#, r#"^\s*def"#, false, || {
         vec!["hello world".to_string()]
@@ -325,7 +348,7 @@ fn test_replace_region_with_start() {
     let text = "\nabc\n123\n789\ndef\nghi";
     let expected = FileChange {
         changed: true,
-        new_lines: "\nhello world\ndef\nghi".to_string()
+        new_lines: "\nhello world\ndef\nghi".to_string(),
     };
     let result = replace_region_in_text(text, r#"^\s*abc$"#, r#"^\s*def"#, true, || {
         vec!["hello world".to_string()]
@@ -338,11 +361,9 @@ fn test_replace_region_no_changes() {
     let text = "123\n456\n789";
     let expected = FileChange {
         changed: false,
-        new_lines: "123\n456\n789".to_string()
+        new_lines: "123\n456\n789".to_string(),
     };
-    let result = replace_region_in_text(text, r#"^\s*123$"#, r#"^\s*456"#, false, || {
-        vec![]
-    });
+    let result = replace_region_in_text(text, r#"^\s*123$"#, r#"^\s*456"#, false, || vec![]);
     assert_eq!(expected, result);
 }
 
@@ -352,11 +373,15 @@ fn test_usable_lints() {
         Lint::new("should_assert_eq", "Deprecated", "abc", Some("Reason"), "module_name"),
         Lint::new("should_assert_eq2", "Not Deprecated", "abc", None, "module_name"),
         Lint::new("should_assert_eq2", "internal", "abc", None, "module_name"),
-        Lint::new("should_assert_eq2", "internal_style", "abc", None, "module_name")
-    ];
-    let expected = vec![
-        Lint::new("should_assert_eq2", "Not Deprecated", "abc", None, "module_name")
+        Lint::new("should_assert_eq2", "internal_style", "abc", None, "module_name"),
     ];
+    let expected = vec![Lint::new(
+        "should_assert_eq2",
+        "Not Deprecated",
+        "abc",
+        None,
+        "module_name",
+    )];
     assert_eq!(expected, Lint::usable_lints(lints.into_iter()).collect::<Vec<Lint>>());
 }
 
@@ -368,13 +393,17 @@ fn test_by_lint_group() {
         Lint::new("incorrect_match", "group1", "abc", None, "module_name"),
     ];
     let mut expected: HashMap<String, Vec<Lint>> = HashMap::new();
-    expected.insert("group1".to_string(), vec![
-        Lint::new("should_assert_eq", "group1", "abc", None, "module_name"),
-        Lint::new("incorrect_match", "group1", "abc", None, "module_name"),
-    ]);
-    expected.insert("group2".to_string(), vec![
-        Lint::new("should_assert_eq2", "group2", "abc", None, "module_name")
-    ]);
+    expected.insert(
+        "group1".to_string(),
+        vec![
+            Lint::new("should_assert_eq", "group1", "abc", None, "module_name"),
+            Lint::new("incorrect_match", "group1", "abc", None, "module_name"),
+        ],
+    );
+    expected.insert(
+        "group2".to_string(),
+        vec![Lint::new("should_assert_eq2", "group2", "abc", None, "module_name")],
+    );
     assert_eq!(expected, Lint::by_lint_group(&lints));
 }
 
@@ -387,7 +416,7 @@ fn test_gen_changelog_lint_list() {
     ];
     let expected = vec![
         format!("[`should_assert_eq`]: {}#should_assert_eq", DOCS_LINK.to_string()),
-        format!("[`should_assert_eq2`]: {}#should_assert_eq2", DOCS_LINK.to_string())
+        format!("[`should_assert_eq2`]: {}#should_assert_eq2", DOCS_LINK.to_string()),
     ];
     assert_eq!(expected, gen_changelog_lint_list(lints));
 }
@@ -395,20 +424,35 @@ fn test_gen_changelog_lint_list() {
 #[test]
 fn test_gen_deprecated() {
     let lints = vec![
-        Lint::new("should_assert_eq", "group1", "abc", Some("has been superseeded by should_assert_eq2"), "module_name"),
-        Lint::new("another_deprecated", "group2", "abc", Some("will be removed"), "module_name"),
-        Lint::new("should_assert_eq2", "group2", "abc", None, "module_name")
+        Lint::new(
+            "should_assert_eq",
+            "group1",
+            "abc",
+            Some("has been superseded by should_assert_eq2"),
+            "module_name",
+        ),
+        Lint::new(
+            "another_deprecated",
+            "group2",
+            "abc",
+            Some("will be removed"),
+            "module_name",
+        ),
+        Lint::new("should_assert_eq2", "group2", "abc", None, "module_name"),
     ];
     let expected: Vec<String> = vec![
         "    store.register_removed(",
         "        \"should_assert_eq\",",
-        "        \"has been superseeded by should_assert_eq2\",",
+        "        \"has been superseded by should_assert_eq2\",",
         "    );",
         "    store.register_removed(",
         "        \"another_deprecated\",",
         "        \"will be removed\",",
-        "    );"
-    ].into_iter().map(String::from).collect();
+        "    );",
+    ]
+    .into_iter()
+    .map(String::from)
+    .collect();
     assert_eq!(expected, gen_deprecated(&lints));
 }
 
