@@ -1,10 +1,10 @@
-use rustc::lint::*;
-use rustc::{declare_lint, lint_array};
-use rustc::hir::*;
-use syntax::ast::{LitKind, NodeId};
-use syntax::codemap::Span;
-use unicode_normalization::UnicodeNormalization;
 use crate::utils::{is_allowed, snippet, span_help_and_lint};
+use rustc::hir::*;
+use rustc::lint::{LateContext, LateLintPass, LintArray, LintPass};
+use rustc::{declare_tool_lint, lint_array};
+use syntax::ast::{LitKind, NodeId};
+use syntax::source_map::Span;
+use unicode_normalization::UnicodeNormalization;
 
 /// **What it does:** Checks for the Unicode zero-width space in the code.
 ///
@@ -38,8 +38,7 @@ declare_clippy_lint! {
 declare_clippy_lint! {
     pub NON_ASCII_LITERAL,
     pedantic,
-    "using any literal non-ASCII chars in a string literal instead of \
-     using the `\\u` escape"
+    "using any literal non-ASCII chars in a string literal instead of using the `\\u` escape"
 }
 
 /// **What it does:** Checks for string literals that contain Unicode in a form
@@ -56,11 +55,9 @@ declare_clippy_lint! {
 declare_clippy_lint! {
     pub UNICODE_NOT_NFC,
     pedantic,
-    "using a unicode literal not in NFC normal form (see \
-     [unicode tr15](http://www.unicode.org/reports/tr15/) for further information)"
+    "using a unicode literal not in NFC normal form (see [unicode tr15](http://www.unicode.org/reports/tr15/) for further information)"
 }
 
-
 #[derive(Copy, Clone)]
 pub struct Unicode;
 
@@ -68,6 +65,10 @@ impl LintPass for Unicode {
     fn get_lints(&self) -> LintArray {
         lint_array!(ZERO_WIDTH_SPACE, NON_ASCII_LITERAL, UNICODE_NOT_NFC)
     }
+
+    fn name(&self) -> &'static str {
+        "Unicode"
+    }
 }
 
 impl<'a, 'tcx> LateLintPass<'a, 'tcx> for Unicode {
@@ -130,7 +131,10 @@ fn check_str(cx: &LateContext<'_, '_>, span: Span, id: NodeId) {
             UNICODE_NOT_NFC,
             span,
             "non-nfc unicode sequence detected",
-            &format!("Consider replacing the string with:\n\"{}\"", string.nfc().collect::<String>()),
+            &format!(
+                "Consider replacing the string with:\n\"{}\"",
+                string.nfc().collect::<String>()
+            ),
         );
     }
 }
