@@ -4,11 +4,12 @@ use std::rc::Rc;
 
 use rustc::ty::{self, layout::Size};
 use rustc::hir::{Mutability, MutMutable, MutImmutable};
+use rustc::mir::RetagKind;
 
 use crate::{
     EvalResult, EvalErrorKind, MiriEvalContext, HelpersEvalContextExt, Evaluator, MutValueVisitor,
     MemoryKind, MiriMemoryKind, RangeMap, AllocId, Allocation, AllocationExtra,
-    Pointer, MemPlace, Scalar, Immediate, ImmTy, PlaceTy, MPlaceTy,
+    Pointer, Immediate, ImmTy, PlaceTy, MPlaceTy,
 };
 
 pub type Timestamp = u64;
@@ -20,19 +21,19 @@ pub type CallId = u64;
 pub enum Borrow {
     /// A unique (mutable) reference.
     Uniq(Timestamp),
-    /// A shared reference.  This is also used by raw pointers, which do not track details
+    /// An aliasing reference.  This is also used by raw pointers, which do not track details
     /// of how or when they were created, hence the timestamp is optional.
     /// Shr(Some(_)) does NOT mean that the destination of this reference is frozen;
     /// that depends on the type!  Only those parts outside of an `UnsafeCell` are actually
     /// frozen.
-    Shr(Option<Timestamp>),
+    Alias(Option<Timestamp>),
 }
 
 impl Borrow {
     #[inline(always)]
-    pub fn is_shared(self) -> bool {
+    pub fn is_aliasing(self) -> bool {
         match self {
-            Borrow::Shr(_) => true,
+            Borrow::Alias(_) => true,
             _ => false,
         }
     }
@@ -48,7 +49,7 @@ impl Borrow {
 
 impl Default for Borrow {
     fn default() -> Self {
-        Borrow::Shr(None)
+        Borrow::Alias(None)
     }
 }
 
@@ -57,16 +58,15 @@ impl Default for Borrow {
 pub enum BorStackItem {
     /// Indicates the unique reference that may mutate.
     Uniq(Timestamp),
-    /// Indicates that the location has been shared.  Used for raw pointers, but
-    /// also for shared references.  The latter *additionally* get frozen
-    /// when there is no `UnsafeCell`.
-    Shr,
+    /// Indicates that the location has been mutably shared.  Used for raw pointers as
+    /// well as for unfrozen shared references.
+    Raw,
     /// A barrier, tracking the function it belongs to by its index on the call stack
     FnBarrier(CallId)
 }
 
 /// Extra per-location state
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Stack {
     borrows: Vec<BorStackItem>, // used as a stack; never empty
     frozen_since: Option<Timestamp>, // virtual frozen "item" on top of the stack
@@ -185,19 +185,19 @@ impl<'tcx> Stack {
         kind: RefKind,
     ) -> Result<Option<usize>, String> {
         // Exclude unique ref with frozen tag.
-        if let (RefKind::Unique, Borrow::Shr(Some(_))) = (kind, bor) {
+        if let (RefKind::Unique, Borrow::Alias(Some(_))) = (kind, bor) {
             return Err(format!("Encountered mutable reference with frozen tag ({:?})", bor));
         }
         // Checks related to freezing
         match bor {
-            Borrow::Shr(Some(bor_t)) if kind == RefKind::Frozen => {
+            Borrow::Alias(Some(bor_t)) if kind == RefKind::Frozen => {
                 // We need the location to be frozen. This ensures F3.
                 let frozen = self.frozen_since.map_or(false, |itm_t| itm_t <= bor_t);
                 return if frozen { Ok(None) } else {
                     Err(format!("Location is not frozen long enough"))
                 }
             }
-            Borrow::Shr(_) if self.frozen_since.is_some() => {
+            Borrow::Alias(_) if self.frozen_since.is_some() => {
                 return Ok(None) // Shared deref to frozen location, looking good
             }
             _ => {} // Not sufficient, go on looking.
@@ -209,8 +209,8 @@ impl<'tcx> Stack {
                     // Found matching unique item.  This satisfies U3.
                     return Ok(Some(idx))
                 }
-                (BorStackItem::Shr, Borrow::Shr(_)) => {
-                    // Found matching shared/raw item.
+                (BorStackItem::Raw, Borrow::Alias(_)) => {
+                    // Found matching aliasing/raw item.
                     return Ok(Some(idx))
                 }
                 // Go on looking.  We ignore barriers!  When an `&mut` and an `&` alias,
@@ -220,7 +220,7 @@ impl<'tcx> Stack {
             }
         }
         // If we got here, we did not find our item.  We have to error to satisfy U3.
-        Err(format!("Borrow being dereferenced ({:?}) does not exist on the stack", bor))
+        Err(format!("Borrow being dereferenced ({:?}) does not exist on the borrow stack", bor))
     }
 
     /// Perform an actual memory access using `bor`.  We do not know any types here
@@ -257,14 +257,15 @@ impl<'tcx> Stack {
                 (BorStackItem::Uniq(itm_t), Borrow::Uniq(bor_t)) if itm_t == bor_t => {
                     // Found matching unique item.  Continue after the match.
                 }
-                (BorStackItem::Shr, _) if kind == AccessKind::Read => {
-                    // When reading, everything can use a shared item!
+                (BorStackItem::Raw, _) if kind == AccessKind::Read => {
+                    // When reading, everything can use a raw item!
                     // We do not want to do this when writing: Writing to an `&mut`
                     // should reaffirm its exclusivity (i.e., make sure it is
-                    // on top of the stack).  Continue after the match.
+                    // on top of the stack).
+                    // Continue after the match.
                 }
-                (BorStackItem::Shr, Borrow::Shr(_)) => {
-                    // Found matching shared item.  Continue after the match.
+                (BorStackItem::Raw, Borrow::Alias(_)) => {
+                    // Found matching raw item.  Continue after the match.
                 }
                 _ => {
                     // Pop this, go on.  This ensures U2.
@@ -293,7 +294,7 @@ impl<'tcx> Stack {
         }
         // If we got here, we did not find our item.
         err!(MachineError(format!(
-            "Borrow being accessed ({:?}) does not exist on the stack",
+            "Borrow being accessed ({:?}) does not exist on the borrow stack",
             bor
         )))
     }
@@ -303,37 +304,42 @@ impl<'tcx> Stack {
     /// is met: We cannot push `Uniq` onto frozen stacks.
     /// `kind` indicates which kind of reference is being created.
     fn create(&mut self, bor: Borrow, kind: RefKind) {
-        if self.frozen_since.is_some() {
-            // A frozen location?  Possible if we create a barrier, then push again.
-            assert!(bor.is_shared(), "We should never try creating a unique borrow for a frozen stack");
-            trace!("create: Not doing anything on frozen location");
+        // When creating a frozen reference, freeze.  This ensures F1.
+        // We also do *not* push anything else to the stack, making sure that no nother kind
+        // of access (like writing through raw pointers) is permitted.
+        if kind == RefKind::Frozen {
+            let bor_t = match bor {
+                Borrow::Alias(Some(t)) => t,
+                _ => bug!("Creating illegal borrow {:?} for frozen ref", bor),
+            };
+            // It is possible that we already are frozen (e.g. if we just pushed a barrier,
+            // the redundancy check would not have kicked in).
+            match self.frozen_since {
+                Some(loc_t) => assert!(loc_t <= bor_t, "Trying to freeze location for longer than it was already frozen"),
+                None => {
+                    trace!("create: Freezing");
+                    self.frozen_since = Some(bor_t);
+                }
+            }
             return;
         }
-        // First, push.  We do this even if we will later freeze, because we
-        // will allow mutation of shared data at the expense of unfreezing.
+        assert!(self.frozen_since.is_none(), "Trying to create non-frozen reference to frozen location");
+
+        // Push new item to the stack.
         let itm = match bor {
             Borrow::Uniq(t) => BorStackItem::Uniq(t),
-            Borrow::Shr(_) => BorStackItem::Shr,
+            Borrow::Alias(_) => BorStackItem::Raw,
         };
         if *self.borrows.last().unwrap() == itm {
             // This is just an optimization, no functional change: Avoid stacking
             // multiple `Shr` on top of each other.
-            assert!(bor.is_shared());
+            assert!(bor.is_aliasing());
             trace!("create: Sharing a shared location is a NOP");
         } else {
             // This ensures U1.
             trace!("create: Pushing {:?}", itm);
             self.borrows.push(itm);
         }
-        // Then, maybe freeze.  This is part 2 of ensuring F1.
-        if kind == RefKind::Frozen {
-            let bor_t = match bor {
-                Borrow::Shr(Some(t)) => t,
-                _ => bug!("Creating illegal borrow {:?} for frozen ref", bor),
-            };
-            trace!("create: Freezing");
-            self.frozen_since = Some(bor_t);
-        }
     }
 
     /// Add a barrier
@@ -434,7 +440,7 @@ impl<'tcx> Stacks {
                     _ => false,
                 };
             if bor_redundant {
-                assert!(new_bor.is_shared(), "A unique reborrow can never be redundant");
+                assert!(new_bor.is_aliasing(), "A unique reborrow can never be redundant");
                 trace!("reborrow is redundant");
                 continue;
             }
@@ -459,7 +465,7 @@ impl AllocationExtra<Borrow, MemoryState> for Stacks {
     #[inline(always)]
     fn memory_allocated<'tcx>(size: Size, extra: &MemoryState) -> Self {
         let stack = Stack {
-            borrows: vec![BorStackItem::Shr],
+            borrows: vec![BorStackItem::Raw],
             frozen_since: None,
         };
         Stacks {
@@ -505,64 +511,98 @@ impl<'tcx> Stacks {
     ) {
         for stack in self.stacks.get_mut().iter_mut(Size::ZERO, size) {
             assert!(stack.borrows.len() == 1);
-            assert_eq!(stack.borrows.pop().unwrap(), BorStackItem::Shr);
+            assert_eq!(stack.borrows.pop().unwrap(), BorStackItem::Raw);
             stack.borrows.push(itm);
         }
     }
 }
 
-
-
-pub trait EvalContextExt<'tcx> {
-    fn ptr_dereference(
-        &self,
-        place: MPlaceTy<'tcx, Borrow>,
-        size: Size,
-        mutability: Option<Mutability>,
-    ) -> EvalResult<'tcx>;
-
-    fn tag_new_allocation(
-        &mut self,
-        id: AllocId,
-        kind: MemoryKind<MiriMemoryKind>,
-    ) -> Borrow;
-
-    /// Reborrow the given place, returning the newly tagged ptr to it.
+impl<'a, 'mir, 'tcx> EvalContextPrivExt<'a, 'mir, 'tcx> for crate::MiriEvalContext<'a, 'mir, 'tcx> {}
+trait EvalContextPrivExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a, 'mir, 'tcx> {
     fn reborrow(
         &mut self,
         place: MPlaceTy<'tcx, Borrow>,
         size: Size,
         fn_barrier: bool,
         new_bor: Borrow
-    ) -> EvalResult<'tcx, Pointer<Borrow>>;
+    ) -> EvalResult<'tcx> {
+        let this = self.eval_context_mut();
+        let ptr = place.ptr.to_ptr()?;
+        let barrier = if fn_barrier { Some(this.frame().extra) } else { None };
+        trace!("reborrow: Creating new reference for {:?} (pointee {}): {:?}",
+            ptr, place.layout.ty, new_bor);
+
+        // Get the allocation.  It might not be mutable, so we cannot use `get_mut`.
+        let alloc = this.memory().get(ptr.alloc_id)?;
+        alloc.check_bounds(this, ptr, size)?;
+        // Update the stacks.
+        if let Borrow::Alias(Some(_)) = new_bor {
+            // Reference that cares about freezing. We need a frozen-sensitive reborrow.
+            this.visit_freeze_sensitive(place, size, |cur_ptr, size, frozen| {
+                let kind = if frozen { RefKind::Frozen } else { RefKind::Raw };
+                alloc.extra.reborrow(cur_ptr, size, barrier, new_bor, kind)
+            })?;
+        } else {
+            // Just treat this as one big chunk.
+            let kind = if new_bor.is_unique() { RefKind::Unique } else { RefKind::Raw };
+            alloc.extra.reborrow(ptr, size, barrier, new_bor, kind)?;
+        }
+        Ok(())
+    }
 
     /// Retag an indidual pointer, returning the retagged version.
+    /// `mutbl` can be `None` to make this a raw pointer.
     fn retag_reference(
         &mut self,
-        ptr: ImmTy<'tcx, Borrow>,
-        mutbl: Mutability,
+        val: ImmTy<'tcx, Borrow>,
+        mutbl: Option<Mutability>,
         fn_barrier: bool,
-    ) -> EvalResult<'tcx, Immediate<Borrow>>;
+        two_phase: bool,
+    ) -> EvalResult<'tcx, Immediate<Borrow>> {
+        let this = self.eval_context_mut();
+        // We want a place for where the ptr *points to*, so we get one.
+        let place = this.ref_to_mplace(val)?;
+        let size = this.size_and_align_of_mplace(place)?
+            .map(|(size, _)| size)
+            .unwrap_or_else(|| place.layout.size);
+        if size == Size::ZERO {
+            // Nothing to do for ZSTs.
+            return Ok(*val);
+        }
 
-    fn retag(
-        &mut self,
-        fn_entry: bool,
-        place: PlaceTy<'tcx, Borrow>
-    ) -> EvalResult<'tcx>;
+        // Compute new borrow.
+        let time = this.machine.stacked_borrows.increment_clock();
+        let new_bor = match mutbl {
+            Some(MutMutable) => Borrow::Uniq(time),
+            Some(MutImmutable) => Borrow::Alias(Some(time)),
+            None => Borrow::default(),
+        };
 
-    fn escape_to_raw(
-        &mut self,
-        place: MPlaceTy<'tcx, Borrow>,
-        size: Size,
-    ) -> EvalResult<'tcx>;
+        // Reborrow.
+        this.reborrow(place, size, fn_barrier, new_bor)?;
+        let new_place = place.with_tag(new_bor);
+        // Handle two-phase borrows.
+        if two_phase {
+            assert!(mutbl == Some(MutMutable), "two-phase shared borrows make no sense");
+            // We immediately share it, to allow read accesses
+            let two_phase_time = this.machine.stacked_borrows.increment_clock();
+            let two_phase_bor = Borrow::Alias(Some(two_phase_time));
+            this.reborrow(new_place, size, /*fn_barrier*/false, two_phase_bor)?;
+        }
+
+        // Return new ptr.
+        Ok(new_place.to_ref())
+    }
 }
 
-impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
+impl<'a, 'mir, 'tcx> EvalContextExt<'a, 'mir, 'tcx> for crate::MiriEvalContext<'a, 'mir, 'tcx> {}
+pub trait EvalContextExt<'a, 'mir, 'tcx: 'a+'mir>: crate::MiriEvalContextExt<'a, 'mir, 'tcx> {
     fn tag_new_allocation(
         &mut self,
         id: AllocId,
         kind: MemoryKind<MiriMemoryKind>,
     ) -> Borrow {
+        let this = self.eval_context_mut();
         let time = match kind {
             MemoryKind::Stack => {
                 // New unique borrow. This `Uniq` is not accessible by the program,
@@ -573,7 +613,7 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
                 // `reset` which the blog post [1] says to perform when accessing a local.
                 //
                 // [1] https://www.ralfj.de/blog/2018/08/07/stacked-borrows.html
-                self.machine.stacked_borrows.increment_clock()
+                this.machine.stacked_borrows.increment_clock()
             }
             _ => {
                 // Nothing to do for everything else
@@ -581,7 +621,7 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
             }
         };
         // Make this the active borrow for this allocation
-        let alloc = self.memory_mut().get_mut(id).expect("This is a new allocation, it must still exist");
+        let alloc = this.memory_mut().get_mut(id).expect("This is a new allocation, it must still exist");
         let size = Size::from_bytes(alloc.bytes.len() as u64);
         alloc.extra.first_item(BorStackItem::Uniq(time), size);
         Borrow::Uniq(time)
@@ -597,6 +637,7 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
         size: Size,
         mutability: Option<Mutability>,
     ) -> EvalResult<'tcx> {
+        let this = self.eval_context_ref();
         trace!("ptr_dereference: Accessing {} reference for {:?} (pointee {})",
             if let Some(mutability) = mutability { format!("{:?}", mutability) } else { format!("raw") },
             place.ptr, place.layout.ty);
@@ -607,13 +648,13 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
         }
 
         // Get the allocation
-        let alloc = self.memory().get(ptr.alloc_id)?;
-        alloc.check_bounds(self, ptr, size)?;
+        let alloc = this.memory().get(ptr.alloc_id)?;
+        alloc.check_bounds(this, ptr, size)?;
         // If we got here, we do some checking, *but* we leave the tag unchanged.
-        if let Borrow::Shr(Some(_)) = ptr.tag {
+        if let Borrow::Alias(Some(_)) = ptr.tag {
             assert_eq!(mutability, Some(MutImmutable));
             // We need a frozen-sensitive check
-            self.visit_freeze_sensitive(place, size, |cur_ptr, size, frozen| {
+            this.visit_freeze_sensitive(place, size, |cur_ptr, size, frozen| {
                 let kind = if frozen { RefKind::Frozen } else { RefKind::Raw };
                 alloc.extra.deref(cur_ptr, size, kind)
             })?;
@@ -627,94 +668,24 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
         Ok(())
     }
 
-    /// The given place may henceforth be accessed through raw pointers.
-    #[inline(always)]
-    fn escape_to_raw(
-        &mut self,
-        place: MPlaceTy<'tcx, Borrow>,
-        size: Size,
-    ) -> EvalResult<'tcx> {
-        self.reborrow(place, size, /*fn_barrier*/ false, Borrow::default())?;
-        Ok(())
-    }
-
-    fn reborrow(
-        &mut self,
-        place: MPlaceTy<'tcx, Borrow>,
-        size: Size,
-        fn_barrier: bool,
-        new_bor: Borrow
-    ) -> EvalResult<'tcx, Pointer<Borrow>> {
-        let ptr = place.ptr.to_ptr()?;
-        let new_ptr = Pointer::new_with_tag(ptr.alloc_id, ptr.offset, new_bor);
-        let barrier = if fn_barrier { Some(self.frame().extra) } else { None };
-        trace!("reborrow: Creating new reference for {:?} (pointee {}): {:?}",
-            ptr, place.layout.ty, new_bor);
-
-        // Get the allocation.  It might not be mutable, so we cannot use `get_mut`.
-        let alloc = self.memory().get(ptr.alloc_id)?;
-        alloc.check_bounds(self, ptr, size)?;
-        // Update the stacks.
-        if let Borrow::Shr(Some(_)) = new_bor {
-            // Reference that cares about freezing. We need a frozen-sensitive reborrow.
-            self.visit_freeze_sensitive(place, size, |cur_ptr, size, frozen| {
-                let kind = if frozen { RefKind::Frozen } else { RefKind::Raw };
-                alloc.extra.reborrow(cur_ptr, size, barrier, new_bor, kind)
-            })?;
-        } else {
-            // Just treat this as one big chunk.
-            let kind = if new_bor.is_unique() { RefKind::Unique } else { RefKind::Raw };
-            alloc.extra.reborrow(ptr, size, barrier, new_bor, kind)?;
-        }
-        Ok(new_ptr)
-    }
-
-    fn retag_reference(
-        &mut self,
-        val: ImmTy<'tcx, Borrow>,
-        mutbl: Mutability,
-        fn_barrier: bool,
-    ) -> EvalResult<'tcx, Immediate<Borrow>> {
-        // We want a place for where the ptr *points to*, so we get one.
-        let place = self.ref_to_mplace(val)?;
-        let size = self.size_and_align_of_mplace(place)?
-            .map(|(size, _)| size)
-            .unwrap_or_else(|| place.layout.size);
-        if size == Size::ZERO {
-            // Nothing to do for ZSTs.
-            return Ok(*val);
-        }
-
-        // Compute new borrow.
-        let time = self.machine.stacked_borrows.increment_clock();
-        let new_bor = match mutbl {
-            MutMutable => Borrow::Uniq(time),
-            MutImmutable => Borrow::Shr(Some(time)),
-        };
-
-        // Reborrow.
-        let new_ptr = self.reborrow(place, size, fn_barrier, new_bor)?;
-
-        // Return new ptr
-        let new_place = MemPlace { ptr: Scalar::Ptr(new_ptr), ..*place };
-        Ok(new_place.to_ref())
-    }
-
     fn retag(
         &mut self,
-        fn_entry: bool,
+        kind: RetagKind,
         place: PlaceTy<'tcx, Borrow>
     ) -> EvalResult<'tcx> {
+        let this = self.eval_context_mut();
         // Determine mutability and whether to add a barrier.
         // Cannot use `builtin_deref` because that reports *immutable* for `Box`,
         // making it useless.
-        fn qualify(ty: ty::Ty<'_>, fn_entry: bool) -> Option<(Mutability, bool)> {
+        fn qualify(ty: ty::Ty<'_>, kind: RetagKind) -> Option<(Option<Mutability>, bool)> {
             match ty.sty {
                 // References are simple
-                ty::Ref(_, _, mutbl) => Some((mutbl, fn_entry)),
+                ty::Ref(_, _, mutbl) => Some((Some(mutbl), kind == RetagKind::FnEntry)),
+                // Raw pointers need to be enabled
+                ty::RawPtr(..) if kind == RetagKind::Raw => Some((None, false)),
                 // Boxes do not get a barrier: Barriers reflect that references outlive the call
                 // they were passed in to; that's just not the case for boxes.
-                ty::Adt(..) if ty.is_box() => Some((MutMutable, false)),
+                ty::Adt(..) if ty.is_box() => Some((Some(MutMutable), false)),
                 _ => None,
             }
         }
@@ -722,22 +693,22 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
         // We need a visitor to visit all references.  However, that requires
         // a `MemPlace`, so we have a fast path for reference types that
         // avoids allocating.
-        if let Some((mutbl, barrier)) = qualify(place.layout.ty, fn_entry) {
+        if let Some((mutbl, barrier)) = qualify(place.layout.ty, kind) {
             // fast path
-            let val = self.read_immediate(self.place_to_op(place)?)?;
-            let val = self.retag_reference(val, mutbl, barrier)?;
-            self.write_immediate(val, place)?;
+            let val = this.read_immediate(this.place_to_op(place)?)?;
+            let val = this.retag_reference(val, mutbl, barrier, kind == RetagKind::TwoPhase)?;
+            this.write_immediate(val, place)?;
             return Ok(());
         }
-        let place = self.force_allocation(place)?;
+        let place = this.force_allocation(place)?;
 
-        let mut visitor = RetagVisitor { ecx: self, fn_entry };
+        let mut visitor = RetagVisitor { ecx: this, kind };
         visitor.visit_value(place)?;
 
         // The actual visitor
         struct RetagVisitor<'ecx, 'a, 'mir, 'tcx> {
             ecx: &'ecx mut MiriEvalContext<'a, 'mir, 'tcx>,
-            fn_entry: bool,
+            kind: RetagKind,
         }
         impl<'ecx, 'a, 'mir, 'tcx>
             MutValueVisitor<'a, 'mir, 'tcx, Evaluator<'tcx>>
@@ -756,9 +727,14 @@ impl<'a, 'mir, 'tcx> EvalContextExt<'tcx> for MiriEvalContext<'a, 'mir, 'tcx> {
             {
                 // Cannot use `builtin_deref` because that reports *immutable* for `Box`,
                 // making it useless.
-                if let Some((mutbl, barrier)) = qualify(place.layout.ty, self.fn_entry) {
+                if let Some((mutbl, barrier)) = qualify(place.layout.ty, self.kind) {
                     let val = self.ecx.read_immediate(place.into())?;
-                    let val = self.ecx.retag_reference(val, mutbl, barrier)?;
+                    let val = self.ecx.retag_reference(
+                        val,
+                        mutbl,
+                        barrier,
+                        self.kind == RetagKind::TwoPhase
+                    )?;
                     self.ecx.write_immediate(val, place.into())?;
                 }
                 Ok(())
