@@ -1,5 +1,3 @@
-#![allow(unused_variables)]
-
 // This makes a ref that was passed to us via &mut alias with things it should not alias with
 fn retarget(x: &mut &u32, target: &mut u32) {
     unsafe { *x = &mut *(target as *mut _); }
@@ -11,5 +9,5 @@ fn main() {
     retarget(&mut target_alias, target);
     // now `target_alias` points to the same thing as `target`
     *target = 13;
-    let _val = *target_alias; //~ ERROR should be frozen
+    let _val = *target_alias; //~ ERROR does not exist on the borrow stack
 }
