@@ -8,12 +8,14 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use config::{Color, Config, Verbosity};
-use diff;
 use std::collections::VecDeque;
 use std::io;
 use std::io::Write;
 
+use diff;
+
+use crate::config::{Color, Config, Verbosity};
+
 #[derive(Debug, PartialEq)]
 pub enum DiffLine {
     Context(String),
@@ -44,7 +46,7 @@ impl Mismatch {
 // This struct handles writing output to stdout and abstracts away the logic
 // of printing in color, if it's possible in the executing environment.
 pub struct OutputWriter {
-    terminal: Option<Box<term::Terminal<Output = io::Stdout>>>,
+    terminal: Option<Box<dyn term::Terminal<Output = io::Stdout>>>,
 }
 
 impl OutputWriter {
